@@ -7,6 +7,7 @@ import multiprocessing
 import platform
 import shutil
 import sys
+from collections import defaultdict
 from functools import partial
 from pathlib import Path
 from typing import Any
@@ -47,8 +48,8 @@ def parse_mapping_csv(mapping_csv: Path) -> dict:
     return df.set_index("gpas_sample_name")["local_sample_name"].to_dict()
 
 
-def fetch_user_details(access_token, environment: ENVIRONMENTS):
-    """Test API authentication and return user details, otherwise exit"""
+def fetch_user_details(access_token, environment: ENVIRONMENTS) -> dict:
+    """Test API authentication and fetch response from userOrgDtls endpoint"""
     endpoint = (
         ENDPOINTS[environment.value]["HOST"]
         + ENDPOINTS[environment.value]["ORDS_PATH"]
@@ -59,20 +60,16 @@ def fetch_user_details(access_token, environment: ENVIRONMENTS):
         r = httpx.get(endpoint, headers={"Authorization": f"Bearer {access_token}"})
         if not r.is_success:
             r.raise_for_status()
-        result = r.json().get("userOrgDtl", {})[0]
+        result = r.json()
         logging.debug(f"{result=}")
-        user = result.get("userName")
-        organisation = result.get("organisation")
-        date_mask = result.get("maskCollectionDate")  # Expects {"N", "MONTH", "WEEK"}
-        allowed_tags = [d.get("tagName") for d in result.get("tags", {})]
     except httpx.HTTPStatusError as e:
-        status_code = e.response.status_code
-        if status_code == 401:
+        if e.response.status_code == 401:
             raise misc.AuthenticationError(
                 f"Authentication failed, check access token and environment"
             ) from None
         else:
             raise e from None
+    # Query PyPI instead of GitHub? GitHub has hopelessly low unauthenticated rate limits
     # try:
     #     r = httpx.get(
     #         "https://api.github.com/repos/GlobalPathogenAnalysisService/gpas-cli/releases/latest"
@@ -84,7 +81,20 @@ def fetch_user_details(access_token, environment: ENVIRONMENTS):
     #         )
     # except:
     #     pass
-    return user, organisation, allowed_tags, date_mask
+    return result
+
+
+def parse_user_details(result: dict) -> tuple:
+    """Parse response from userOrgDtls endpoint"""
+    result = result.get("userOrgDtl", {})[0]
+    user = result.get("userName")
+    organisation = result.get("organisation")
+    permitted_tags = [
+        d.get("tagName") for d in result.get("tags", {}) if d.get("tagAccessYN") == "Y"
+    ]
+    logging.debug(f"{permitted_tags=}")
+    date_mask = result.get("maskCollectionDate")  # Expects {"N", "MONTH", "WEEK"}
+    return user, organisation, permitted_tags, date_mask
 
 
 def update_fasta_header(path: Path, guid: str, name: str):
@@ -481,12 +491,15 @@ class Batch:
         self.user_agent_name = user_agent_name
         self.user_agent_version = user_agent_version
         if self.token:
+            auth_result = fetch_user_details(
+                self.token["access_token"], self.environment
+            )
             (
                 self.user,
                 self.organisation,
                 self.permitted_tags,
                 self.date_mask,
-            ) = fetch_user_details(self.token["access_token"], self.environment)
+            ) = parse_user_details(auth_result)
             self.headers = {
                 "Authorization": f"Bearer {self.token['access_token']}",
                 "Content-Type": "application/json",
@@ -603,9 +616,17 @@ class Batch:
         result = r.json()
         logging.debug(f"{result=}")
         self.batch_guid = result["batch"]["guid"]
-        hashes_guids = {s["hash"]: s["guid"] for s in result["batch"]["samples"]}
+        guids_hashes = {s["guid"]: s["hash"] for s in result["batch"]["samples"]}
+        logging.debug(f"{guids_hashes=}")
+
+        # Used hash-keyed dict of lists to tolerate samples with same hash
+        hashes_guids = defaultdict(list)
+        for g, h in guids_hashes.items():
+            hashes_guids[h].append(g)
+
+        # Assign each sample a guid from the pool associated with each hash
         for sample in self.samples:
-            sample.guid = hashes_guids[getattr(sample, md5_attr)]
+            sample.guid = hashes_guids[getattr(sample, md5_attr)].pop()
 
     def _rename_fastqs(self):
         """Rename decontaminated fastqs using server-side guids"""
@@ -744,9 +765,7 @@ class Batch:
             + "batches"
         )
 
-        self.submission["batch"]["uploader"][
-            "upload_finish_time"
-        ] = misc.oracle_timestamp()
+        self.submission["batch"]["uploader"]["upload_finish"] = misc.oracle_timestamp()
         logging.debug(json.dumps(self.submission, indent=4))
         r = httpx.post(
             url=endpoint,
