@@ -292,7 +292,7 @@ def get_reference_path(organism):
 
 
 @retry(
-    retry=retry_if_exception_type(httpx.TransportError),
+    retry=retry_if_exception_type(httpx.RequestError),
     wait=wait_exponential(multiplier=1, min=1, max=5),
     stop=stop_after_attempt(5),
 )
